
/*
** Library name:                 pic
** This header file name:        pic.h
** Other necessary libraries:    none
** Other necessary header files: none
**
**          AUTORZY  :  Leszek Chmielewski
**                       - powiedzmy, ogolna koncepcje i duza czesc kodu
**                      Pawel Kies
**                       - duzo roznych rzeczy, a szczegolnie koncepcja
**                         struktury pamieci obrazu dla szybkiego dostepu
**                      Adam Kurianski
**                       - obsluga plikow obrazu w standardzie TIFF
**      NAZWA PLIKU  : pic.h
**
**     PRZEZNACZENIE : Biblioteka podstawowych operacji na reprezentacjach
**                     obrazow. Nie obejmuje grafiki!
**
**   Oprogramowanie powstalo w drodze modyfikacji i rozszerzenia biblioteki
** LCPIC.LIB (patrz opis w pliku lcpic.h), ktore wykonano w ramach projektu
** COPERNICUS CRASH.
**   Projekt zostal zrealizowany w Towarzystwie Przetwarzania Obrazow.
**
**  REMARKS:
**
**  A. This file contains only basic definitions. Much more can be found
**     in the following files, which are included together with pic.h:
**   0. picaux.h   - auxiliary, private defs, not to be utilized by a user.
**   1. picdiag.h  - getting info about pictures and windows.
**   2. picsetal.h - setting all picture or its window to a same value.
**   3. picpix.h   - accessing single pixels of a picture.
**   4. picnbr.h   - quick access to neighbours of a pixel.
**   5. piccpy.h   - duplicating the pictures.
**   6. picscal.h  - scaling (resizing) pictures and windows.
**
**  B. This software differs a lot from that described in lcpic.h. Mainly:
**   1. Now the pixel coordinates are x,y,z instead of R,C,S
**      (row, clm=column, slc=slice). x corresponds to C, y to R and z to S.
**      Note that the typical order of coordinates in parameter lists has
**      changed: x,y,z instead R,C,S, which would correspond to y,x,z.
**   2. The way pictures are stored is improved for speed and better
**      compatibility with the X11 graphic standard.
**   3. Many new functions and macros have been added and many old names have
**      been changed into more consistent ones.
**   4. Size (Z=siz) of images, windows etc. is usually used instead of
**      upper value (U=Upr).
**   5. Definition of a window has changed.
**
*/
#if !defined(PIC_INCLUDED)
  #define PIC_INCLUDED

#include <string.h>

#ifdef __cplusplus
extern "C" {
#endif

/* error codes returned by functions */
#define OK                            0
#define ERROR                        -1
#define BAD_DATA                     -2
#define NO_MEMORY                    -3
#define WRITE_ERROR                  -4
#define READ_ERROR                   -5
#define COORDINATE_DEGENERATE        -6
/* no other error codes inherited from anywhere */

/* function returning pointers to strings with error descriptions */
char * picErrStr(int err);


/* PICTURES */

/* implemented picture modes (or types) (signed char) */
#define BINpic    1  /* binary: 1 bit/pixel, 8 pixels stored in byte */
#define BINpicV  10  /* volume binary */
#define BIT2pic   2  /* 2-bit: 2 bits/pixel, 4 pixels stored in byte */
#define BIT4pic   4  /* 4-bit: 4 bits/pixel, 2 pixels stored in byte */
#define GREpic    8  /* grey: 8 bits/pixel, 1 pixel stored in byte */
#define GREpicV  18  /* volume grey */
#define CLRpic    3  /* colour: 3 bytes/pixel */

/* colour components */
#define Rclr  0
#define Gclr  1
#define Bclr  2
/* smallest (loWer) colour component index */
#define Wclr 0
/* largest  (Upper) colour component index = no.of colours-1 */
#define Uclr 2

typedef unsigned char Tbrh; /* type of brightness in one grey pixel: 1 byte */
#define Xbrh 255        /* maximum value of brightness */
#define Ubrh 255        /* upper value of brightness - whichever you prefer */
#define Mbrh   0        /* minimum value of brightness */
#define Wbrh   0        /* lower value of brightness */
typedef Tbrh Tclr[3];       /* type of brh of three colours in pix: table */

/* type of the picture structure */
typedef struct {
  signed char mod; /* picture mode: BINpic, GREpic etc. */
  signed char isAlloc; /* is pixel array allocated */
  int Zx; /* siZe along x coordinate (columns); Zx = Ux+1; x = 0, ..., Ux */
  int Zy; /* siZe along y coordinate (rows) */
  int Zz; /* siZe along z coordinate (slices) */
  int Zpix; /* siZe of storage of one pixel: colour - 3, others - 1 */
  int Zrow; /* siZe of storage of one row: Zx * Zpix */
  unsigned int Zpic; /* siZe of storage of the whole picture */
  char *Pdat; /* pointer to data; their structure depends on mod */
} Tpic;


/* flat window: of a picture, of a screen, ... */
typedef struct {
  int Mx,My,Zx,Zy; /* in other words: top,left,width,height */
} Twin;

/* volumetric window of a 3D picture */
typedef struct {
  int Mx,My,Mz,Zx,Zy,Zz;
} TwinV;

/* lower right corner position - reading and setting */
#define _Mx(Pwin)  ( (Pwin)->Mx )
#define _My(Pwin)  ( (Pwin)->My )
#define _Mz(Pwin)  ( (Pwin)->Mz )
#define _Xx(Pwin)  ( (Pwin)->Mx + (Pwin)->Zx - 1 )
#define _Xy(Pwin)  ( (Pwin)->My + (Pwin)->Zy - 1 )
#define _Xz(Pwin)  ( (Pwin)->Mz + (Pwin)->Zz - 1 )
#define SetMx(Pwin,v)  ( (Pwin)->Mx = (v) )
#define SetMy(Pwin,v)  ( (Pwin)->My = (v) )
#define SetMz(Pwin,v)  ( (Pwin)->Mz = (v) )
#define SetXx(Pwin,v)  ( (Pwin)->Zx = (v) - (Pwin)->Mx + 1 )
#define SetXy(Pwin,v)  ( (Pwin)->Zy = (v) - (Pwin)->My + 1 )
#define SetXz(Pwin,v)  ( (Pwin)->Zz = (v) - (Pwin)->Mz + 1 )
#define SetZx(Pwin,v)  ( (Pwin)->Zx = (v) )
#define SetZy(Pwin,v)  ( (Pwin)->Zy = (v) )
#define SetZz(Pwin,v)  ( (Pwin)->Zz = (v) )


/* Initializing, allocation and deallocation */

/* Initializes picture structure before the first use, sets mod to NONpic etc. */
void InitPic(Tpic *Ppic);

/* Allocates memory for picture; returns OK,BAD_DATA,NO_MEMORY */
int AllocPic(Tpic *Ppic,int mod,int Zx,int Zy,...);
/* additional argument int Zz needed for mode GREpicV, BINpicV, for example:
** err = AllocPic(&pic1,GREpicV,Zx,Zy,Zz); */

/* Assigns necessary auxiliary data to a picture, the pixels of which have
** already been allocated elsewhere, e.g. by a graphic environment. It is
** assumed that pixels are 1 byte each, ordered by: colours (R,G,B), then
** rows, and then slices. Returns OK,BAD_DATA,NO_MEMORY */
int AssignPic(Tpic *Ppic,Tbrh *PpixA,int mod,int Zx,int Zy,...);

/* Yields the pixel array, so it won't be freed.
** (Picture formed by AllocPic() will be treated as formed by AssignPic() */
#define YieldPpixA(Ppic) ((Ppic)->isAlloc=0)

/* Frees memory of picture and re-initializes it. If picture has been an
** assigned picture, the array of pixels is not freed. */
void FreePic(Tpic *Ppic);


#include "picaux.h"
#include "picdiag.h"
#include "picsetal.h"
#include "picpix.h"
#include "picnbr.h"
#include "piccpy.h"
#include "picscal.h"


#ifdef __cplusplus
}
#endif

#endif /* !defined(PIC_INCLUDED) */
